@@ -286,9 +286,14 @@
 #   include <sys/un.h>
 #   include <sys/uio.h>             //  Let CZMQ build with libzmq/3.x
 #   include <netinet/in.h>          //  Must come before arpa/inet.h
-#   if (!defined (__UTYPE_ANDROID)) && (!defined (__UTYPE_IBMAIX)) \
-    && (!defined (__UTYPE_HPUX))
-#       include <ifaddrs.h>
+#   if (!defined (__UTYPE_IBMAIX)) && (!defined (__UTYPE_HPUX))
+#       if(defined(__UTYPE_ANDROID))
+#           if(defined(__ANDROID_API__)) && (__ANDROID_API__ >= 24)
+#               include <ifaddrs.h>
+#           endif
+#       else
+#           include <ifaddrs.h>
+#       endif
 #   endif
 #   if defined (__UTYPE_SUNSOLARIS) || defined (__UTYPE_SUNOS)
 #       include <sys/sockio.h>
@@ -541,7 +546,7 @@ typedef struct {
 #   endif
 #   define srandom srand
 #   define TIMEZONE _timezone
-#   if (!defined (__MINGW32__))
+#   if (!defined (__MINGW32__)) && (defined(_MSC_VER) && _MSC_VER < 1500)
 #       define snprintf _snprintf
 #       define vsnprintf _vsnprintf
 #   endif
