@@ -376,6 +376,9 @@ zchunk_slurp (const char *filename, size_t maxsize)
         size = maxsize;
 
     FILE *handle = fopen (filename, "r");
+    if (!handle)
+        return NULL;
+
     zchunk_t *chunk = zchunk_read (handle, size);
     assert (chunk);
     fclose (handle);
@@ -474,7 +477,7 @@ zchunk_pack (zchunk_t *self)
 {
     assert (self);
     assert (zchunk_is (self));
-    return zframe_new (self->data, self->max_size);
+    return zframe_new (self->data, self->size);
 }
 
 
@@ -502,7 +505,7 @@ zchunk_packx (zchunk_t **self_p) {
     zchunk_t *self = *self_p;
     *self_p = NULL;
 
-    return zframe_frommem (self->data, self->max_size, (zchunk_destructor_fn *) zchunk_destroy, self);
+    return zframe_frommem (self->data, self->size, (zchunk_destructor_fn *) zchunk_destroy, self);
 }
 
 
