@@ -30,6 +30,7 @@ typedef struct {
     char *address;
     char *netmask;
     char *broadcast;
+    bool is_ipv6;
 } interface_t;
 
 
@@ -42,11 +43,11 @@ s_interface_destroy (interface_t **self_p)
     assert (self_p);
     interface_t *self = *self_p;
     if (self) {
-        free (self->name);
-        free (self->address);
-        free (self->netmask);
-        free (self->broadcast);
-        free (self);
+        freen (self->name);
+        freen (self->address);
+        freen (self->netmask);
+        freen (self->broadcast);
+        freen (self);
         *self_p = NULL;
     }
 }
@@ -56,18 +57,60 @@ s_interface_destroy (interface_t **self_p)
 //  interface constructor
 
 static interface_t *
-s_interface_new (char *name, inaddr_t address, inaddr_t netmask, inaddr_t broadcast)
+s_interface_new (char *name, struct sockaddr *address, struct sockaddr *netmask,
+        struct sockaddr *broadcast)
 {
+    char hbuf[NI_MAXHOST];
+    int rc;
+
     interface_t *self = (interface_t *) zmalloc (sizeof (interface_t));
     assert (self);
     self->name = strdup (name);
     assert (self->name);
-    self->address = strdup (inet_ntoa (address.sin_addr));
+
+    rc = getnameinfo (address, address->sa_family == AF_INET ?
+            sizeof (struct sockaddr_in) : sizeof (struct sockaddr_in6),
+            hbuf, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+    assert (rc == 0);
+    //  Some platform's getnameinfo, like Solaris, appear not to append the
+    //  interface name when parsing a link-local IPv6 address. These addresses
+    //  cannot be used without the interface, so we must append it manually.
+    if (address->sa_family == AF_INET6 &&
+            IN6_IS_ADDR_LINKLOCAL (&((struct sockaddr_in6 *)address)->sin6_addr) &&
+            !strchr (hbuf, '%')) {
+        strcat (hbuf, "%");
+        strcat (hbuf, name);
+    }
+    self->address = strdup (hbuf);
     assert (self->address);
-    self->netmask = strdup (inet_ntoa (netmask.sin_addr));
+
+    rc = getnameinfo (netmask, netmask->sa_family == AF_INET ?
+            sizeof (struct sockaddr_in) : sizeof (struct sockaddr_in6),
+            hbuf, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+    assert (rc == 0);
+    self->netmask = strdup (hbuf);
     assert (self->netmask);
-    self->broadcast = strdup (inet_ntoa (broadcast.sin_addr));
-    assert (self->broadcast);
+
+    if (address->sa_family == AF_INET) {
+        //  If the returned broadcast address is the same as source
+        //  address, build the broadcast address from the source
+        //  address and netmask.
+        if (((inaddr_t *)address)->sin_addr.s_addr == ((inaddr_t *)broadcast)->sin_addr.s_addr)
+            ((inaddr_t *)broadcast)->sin_addr.s_addr |= ~(((inaddr_t *)netmask)->sin_addr.s_addr);
+
+        rc = getnameinfo (broadcast, sizeof (struct sockaddr_in), hbuf, NI_MAXHOST,
+                NULL, 0, NI_NUMERICHOST);
+        assert (rc == 0);
+        self->broadcast = strdup (hbuf);
+        assert (self->broadcast);
+    } else {
+        //  The default is link-local all-node multicast group fe02::1
+        self->broadcast = strdup (zsys_ipv6_mcast_address ());
+        assert (self->broadcast);
+    }
+
+    self->is_ipv6 = address->sa_family == AF_INET6 ? true : false;
+
     return self;
 }
 
@@ -87,6 +130,20 @@ ziflist_new (void)
     return self;
 }
 
+//  --------------------------------------------------------------------------
+//  Get a list of network interfaces currently defined on the system
+//  Includes IPv6 interfaces
+
+ziflist_t *
+ziflist_new_ipv6 (void)
+{
+    ziflist_t *self = (ziflist_t *) zlistx_new ();
+    assert (self);
+    zlistx_set_destructor ((zlistx_t *) self, (czmq_destructor *) s_interface_destroy);
+    ziflist_reload_ipv6 (self);
+    return self;
+}
+
 
 //  Print properties of the ziflist object.
 //  --------------------------------------------------------------------------
@@ -121,11 +178,12 @@ ziflist_destroy (ziflist_t **self_p)
 //  Helper function to verify if one interface's flags are what we want.
 
 static bool
-s_valid_flags (short flags)
+s_valid_flags (short flags, bool ipv6)
 {
     return (flags & IFF_UP)             //  Only use interfaces that are running
            && !(flags & IFF_LOOPBACK)   //  Ignore loopback interface
-           && (flags & IFF_BROADCAST)   //  Only use interfaces that have BROADCAST
+           && ((ipv6 || (flags & IFF_BROADCAST))  //  Only use interfaces that have BROADCAST
+                   && (!ipv6 || (flags & IFF_MULTICAST))) //  or IPv6 and MULTICAST
 #   if defined (IFF_SLAVE)
            && !(flags & IFF_SLAVE)      //  Ignore devices that are bonding slaves.
 #   endif
@@ -135,10 +193,10 @@ s_valid_flags (short flags)
 
 
 //  --------------------------------------------------------------------------
-//  Reload network interfaces from system
+//  Helper to reload network interfaces from system
 
-void
-ziflist_reload (ziflist_t *self)
+static void
+s_reload (ziflist_t *self, bool ipv6)
 {
     assert (self);
     zlistx_t *list = (zlistx_t *) self;
@@ -150,23 +208,19 @@ ziflist_reload (ziflist_t *self)
         struct ifaddrs *interface = interfaces;
         while (interface) {
             //  On Solaris, loopback interfaces have a NULL in ifa_broadaddr
-            if (interface->ifa_broadaddr
-            &&  interface->ifa_addr
-            &&  interface->ifa_addr->sa_family == AF_INET
-            &&  s_valid_flags (interface->ifa_flags)) {
-                inaddr_t address = *(inaddr_t *) interface->ifa_addr;
-                inaddr_t netmask = *(inaddr_t *) interface->ifa_netmask;
-                inaddr_t broadcast = *(inaddr_t *) interface->ifa_broadaddr;
-
-                //  If the returned broadcast address is the same as source
-                //  address, build the broadcast address from the source
-                //  address and netmask.
-                if (address.sin_addr.s_addr == broadcast.sin_addr.s_addr)
-                    broadcast.sin_addr.s_addr |= ~(netmask.sin_addr.s_addr);
-
-                interface_t *item =
-                    s_interface_new (interface->ifa_name, address, netmask,
-                                     broadcast);
+            if (interface->ifa_addr
+            && (interface->ifa_broadaddr
+                    || (ipv6 && (interface->ifa_addr->sa_family == AF_INET6)))
+            &&(interface->ifa_addr->sa_family == AF_INET
+                    || (ipv6 && (interface->ifa_addr->sa_family == AF_INET6)))
+            // Seems to be needed for running VirtualBox VMs on MacOS (see #1802)
+            &&(interface->ifa_netmask->sa_family == AF_INET
+                    || (ipv6 && (interface->ifa_netmask->sa_family == AF_INET6)))
+            &&  s_valid_flags (interface->ifa_flags,
+                    ipv6 && (interface->ifa_addr->sa_family == AF_INET6))) {
+                interface_t *item = s_interface_new (interface->ifa_name,
+                        interface->ifa_addr, interface->ifa_netmask,
+                        interface->ifa_broadaddr);
                 if (item)
                     zlistx_add_end (list, item);
             }
@@ -191,8 +245,6 @@ ziflist_reload (ziflist_t *self)
             struct ifreq *ifr = &ifconfig.ifc_req [index];
             //  Check interface flags
             bool is_valid = false;
-            if (!ioctl (sock, SIOCGIFFLAGS, (caddr_t) ifr, sizeof (struct ifreq)))
-                is_valid = s_valid_flags (ifr->ifr_flags);
 
             //  Get interface properties
             inaddr_t address = { 0 };
@@ -201,6 +253,10 @@ ziflist_reload (ziflist_t *self)
             else
                 is_valid = false;
 
+            if (!ioctl (sock, SIOCGIFFLAGS, (caddr_t) ifr, sizeof (struct ifreq)))
+                is_valid = s_valid_flags (ifr->ifr_flags,
+                        ipv6 && (address.sin_family == AF_INET6));
+
             inaddr_t broadcast = { 0 };
             if (!ioctl (sock, SIOCGIFBRDADDR, (caddr_t) ifr, sizeof (struct ifreq)))
                 broadcast = *((inaddr_t *) &ifr->ifr_addr);
@@ -214,24 +270,45 @@ ziflist_reload (ziflist_t *self)
                 is_valid = false;
 
             if (is_valid) {
-                interface_t *item = s_interface_new (ifr->ifr_name, address,
-                                                     netmask, broadcast);
+                interface_t *item = s_interface_new (ifr->ifr_name,
+                        (struct sockaddr *)&address, (struct sockaddr *)&netmask,
+                        (struct sockaddr *)&broadcast);
                 if (item)
                     zlistx_add_end (list, item);
             }
         }
-        free (ifconfig.ifc_buf);
+        freen (ifconfig.ifc_buf);
         close (sock);
     }
 
 #   elif defined (__WINDOWS__)
+    // TODO: IPv6 support
     ULONG addr_size = 0;
     DWORD rc = GetAdaptersAddresses (AF_INET, GAA_FLAG_INCLUDE_PREFIX, NULL, NULL, &addr_size);
     assert (rc == ERROR_BUFFER_OVERFLOW);
 
-    PIP_ADAPTER_ADDRESSES pip_addresses = (PIP_ADAPTER_ADDRESSES) zmalloc (addr_size);
-    rc = GetAdaptersAddresses (AF_INET,
-                               GAA_FLAG_INCLUDE_PREFIX, NULL, pip_addresses, &addr_size);
+    static const int maxTries = 3;
+    int iterations = 0;
+
+    PIP_ADAPTER_ADDRESSES pip_addresses = NULL;
+
+    do {
+        pip_addresses = (PIP_ADAPTER_ADDRESSES) zmalloc (addr_size);
+        rc = GetAdaptersAddresses (AF_INET,
+                                   GAA_FLAG_INCLUDE_PREFIX, NULL, pip_addresses, &addr_size);
+        
+        if (rc == ERROR_BUFFER_OVERFLOW) {
+            freen (pip_addresses);
+            pip_addresses = NULL;
+        } 
+        else {
+            break;
+        }
+        
+        ++iterations;
+    }
+    while ((rc == ERROR_BUFFER_OVERFLOW) && (iterations < maxTries));
+
     assert (rc == NO_ERROR);
 
     PIP_ADAPTER_ADDRESSES cur_address = pip_addresses;
@@ -250,20 +327,23 @@ ziflist_reload (ziflist_t *self)
                      && (pPrefix->PrefixLength <= 32);
 
         if (valid) {
-            inaddr_t address = *(inaddr_t *) pUnicast->Address.lpSockaddr;
-            inaddr_t netmask;
+            struct sockaddr_in address, netmask, broadcast;
+            address = *(struct sockaddr_in *)pUnicast->Address.lpSockaddr;
+            address.sin_family = AF_INET;
             netmask.sin_addr.s_addr = htonl ((0xffffffffU) << (32 - pPrefix->PrefixLength));
-            inaddr_t broadcast = address;
+            netmask.sin_family = AF_INET;
+            broadcast = address;
             broadcast.sin_addr.s_addr |= ~(netmask.sin_addr.s_addr);
-            interface_t *item = s_interface_new (asciiFriendlyName, address,
-                                                 netmask, broadcast);
+            interface_t *item = s_interface_new (asciiFriendlyName,
+                    (struct sockaddr *)&address, (struct sockaddr *)&netmask,
+                    (struct sockaddr *)&broadcast);
             if (item)
                 zlistx_add_end (list, item);
         }
-        free (asciiFriendlyName);
+        freen (asciiFriendlyName);
         cur_address = cur_address->Next;
     }
-    free (pip_addresses);
+    freen (pip_addresses);
 
 #   else
 #       error "Interface detection TBD on this operating system"
@@ -271,6 +351,26 @@ ziflist_reload (ziflist_t *self)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Reload network interfaces from system
+
+void
+ziflist_reload (ziflist_t *self)
+{
+    s_reload (self, false);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Reload network interfaces from system, including IPv6
+
+void
+ziflist_reload_ipv6 (ziflist_t *self)
+{
+    s_reload (self, true);
+}
+
+
 //  --------------------------------------------------------------------------
 //  Return the number of network interfaces on system
 
@@ -357,6 +457,18 @@ ziflist_netmask (ziflist_t *self)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Return true if the current interface uses IPv6
+
+bool
+ziflist_is_ipv6 (ziflist_t *self)
+{
+    assert (self);
+    interface_t *iface = (interface_t *) zlistx_item ((zlistx_t *) self);
+    return iface->is_ipv6;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Selftest for this class
 
@@ -367,6 +479,16 @@ ziflist_test (bool verbose)
     if (verbose)
         printf ("\n");
 
+    // TODO: for any Windows dev, any alternative to this?
+#if defined (__WINDOWS__)
+    WORD version_requested = MAKEWORD (2, 2);
+    WSADATA wsa_data;
+    int rc = WSAStartup (version_requested, &wsa_data);
+    assert (rc == 0);
+    assert (LOBYTE (wsa_data.wVersion) == 2 &&
+        HIBYTE (wsa_data.wVersion) == 2);
+#endif
+
     //  @selftest
     ziflist_t *iflist = ziflist_new ();
     assert (iflist);
@@ -385,6 +507,10 @@ ziflist_test (bool verbose)
     ziflist_reload (iflist);
     assert (items == ziflist_size (iflist));
     ziflist_destroy (&iflist);
+
+#if defined (__WINDOWS__)
+    zsys_shutdown();
+#endif
     //  @end
     
     printf ("OK\n");
