@@ -387,28 +387,8 @@ zproxy (zsock_t *pipe, void *unused)
 //  --------------------------------------------------------------------------
 //  Selftest
 
-#if (ZMQ_VERSION_MAJOR == 4)
-
 #define LOCALENDPOINT "tcp://127.0.0.1:%d"
 
-static void
-s_create_test_sockets (zactor_t **proxy, zsock_t **faucet, zsock_t **sink, bool verbose)
-{
-    zsock_destroy (faucet);
-    zsock_destroy (sink);
-    zactor_destroy (proxy);
-    *faucet = zsock_new (ZMQ_PUSH);
-    assert (*faucet);
-    *sink = zsock_new (ZMQ_PULL);
-    assert (*sink);
-    *proxy = zactor_new (zproxy, NULL);
-    assert (*proxy);
-    if (verbose) {
-        zstr_sendx (*proxy, "VERBOSE", NULL);
-        zsock_wait (*proxy);
-    }
-}
-
 static int
 s_get_available_port (void)
 {
@@ -431,6 +411,26 @@ s_get_available_port (void)
     return port_nbr;
 }
 
+#if (ZMQ_VERSION_MAJOR == 4)
+
+static void
+s_create_test_sockets (zactor_t **proxy, zsock_t **faucet, zsock_t **sink, bool verbose)
+{
+    zsock_destroy (faucet);
+    zsock_destroy (sink);
+    zactor_destroy (proxy);
+    *faucet = zsock_new (ZMQ_PUSH);
+    assert (*faucet);
+    *sink = zsock_new (ZMQ_PULL);
+    assert (*sink);
+    *proxy = zactor_new (zproxy, NULL);
+    assert (*proxy);
+    if (verbose) {
+        zstr_sendx (*proxy, "VERBOSE", NULL);
+        zsock_wait (*proxy);
+    }
+}
+
 //  Checks whether client can connect to server
 //  expect_success is used to synchronise the connect as sleeps are unreliable
 static bool
@@ -567,30 +567,19 @@ zproxy_test (bool verbose)
     proxy = zactor_new (zproxy, NULL);
     assert (proxy);
 
-#ifdef  WIN32
-	sink = zsock_new_sub(">inproc://backend", "whatever");
-#else
-    // vagrant vms don't like using shared storage for ipc pipes..
-    if (getenv("USER") && streq(getenv("USER"), "vagrant"))
-        sink = zsock_new_sub (">ipc:///tmp/backend", "whatever");
-    else
-	    sink = zsock_new_sub (">ipc://backend", "whatever");
-#endif //  WIN32
-	assert (sink);
+    char *frontend = NULL;
+    char *backend = NULL;
+    backend = zsys_sprintf (LOCALENDPOINT, s_get_available_port ());
+    zclock_sleep (200);
+    sink = zsock_new_sub (backend, "whatever");
+    assert (sink);
 
-#ifdef WIN32
-	zstr_sendx (proxy, "BACKEND", "XPUB", "inproc://backend", NULL);
-#else
-    // vagrant vms don't like using shared storage for ipc pipes..
-    if (getenv("USER") && streq(getenv("USER"), "vagrant"))
-        zstr_sendx(proxy, "BACKEND", "XPUB", "ipc:///tmp/backend", NULL);
-    else
-        zstr_sendx(proxy, "BACKEND", "XPUB", "ipc://backend", NULL);
-#endif
+    zstr_sendx (proxy, "BACKEND", "XPUB", backend, NULL);
     zsock_wait (proxy);
 
     zsock_destroy(&sink);
     zactor_destroy(&proxy);
+    zstr_free (&backend);
 
 #ifdef CZMQ_BUILD_DRAFT_API
     //  Create and configure our proxy with PUB/SUB to test subscriptions
@@ -649,9 +638,6 @@ zproxy_test (bool verbose)
     //  Create temporary directory for test files
     zsys_dir_create (basedirpath);
 
-    char *frontend = NULL;
-    char *backend = NULL;
-
     //  Check there's no authentication
     s_create_test_sockets (&proxy, &faucet, &sink, verbose);
     s_bind_test_sockets (proxy, &frontend, &backend);
@@ -683,7 +669,7 @@ zproxy_test (bool verbose)
         true);
     assert (success);
 
-    //  Blacklist 127.0.0.1, connection should fail
+    //  Block 127.0.0.1, connection should fail
     zstr_sendx (proxy, "DOMAIN", "FRONTEND", "global", NULL);
     zsock_wait (proxy);
     s_bind_test_sockets (proxy, &frontend, &backend);
@@ -693,7 +679,7 @@ zproxy_test (bool verbose)
         false);
     assert (!success);
 
-    //  Whitelist our address, which overrides the blacklist
+    //  Allow our address, which overrides the block list
     zstr_sendx (proxy, "DOMAIN", "FRONTEND", "global", NULL);
     zsock_wait (proxy);
     zstr_sendx (proxy, "DOMAIN", "BACKEND", "global", NULL);
