@@ -69,6 +69,10 @@ s_handler_fn_shim (DWORD ctrltype)
 //  ZeroMQ context for this process
 static void *s_process_ctx = NULL;
 static bool s_initialized = false;
+static bool s_shutting_down = false;
+#if defined (__UNIX__)
+static void zsys_cleanup (void);
+#endif
 
 #ifndef S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC
 // This is a private tunable that is likely to be replaced or tweaked later
@@ -83,6 +87,7 @@ static size_t s_io_threads = 1;     //  ZSYS_IO_THREADS=1
 static int s_thread_sched_policy = -1; //  ZSYS_THREAD_SCHED_POLICY=-1
 static int s_thread_priority = -1;  //  ZSYS_THREAD_PRIORITY=-1
 static int s_thread_name_prefix = -1;  //  ZSYS_THREAD_NAME_PREFIX=-1
+static char s_thread_name_prefix_str[16] = "0";  //  ZSYS_THREAD_NAME_PREFIX_STR="0"
 static size_t s_max_sockets = 1024; //  ZSYS_MAX_SOCKETS=1024
 static int s_max_msgsz = INT_MAX;   //  ZSYS_MAX_MSGSZ=INT_MAX
 static int64_t s_file_stable_age_msec = S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC;
@@ -101,6 +106,8 @@ static FILE *s_logstream = NULL;    //  ZSYS_LOGSTREAM=stdout/stderr
 static bool s_logsystem = false;    //  ZSYS_LOGSYSTEM=true/false
 static zsock_t *s_logsender = NULL;    //  ZSYS_LOGSENDER=
 static int s_zero_copy_recv = 1;    // ZSYS_ZERO_COPY_RECV=1
+static char *s_ipv4_mcast_address = NULL; //  ZSYS_IPV4_MCAST_ADDRESS=
+static unsigned char s_mcast_ttl = 1;     //  ZSYS_MCAST_TTL=1
 
 //  Track number of open sockets so we can zmq_term() safely
 static size_t s_open_sockets = 0;
@@ -133,7 +140,10 @@ typedef CRITICAL_SECTION zsys_mutex_t;
 
 //  Mutex to guard socket counter
 static zsys_mutex_t s_mutex;
-
+#if defined (__UNIX__)
+// Mutex to guard the multiple zsys_init() - to make it threadsafe
+static zsys_mutex_t s_init_mutex;
+#endif
 //  Implementation for the zsys_vprintf() which is known from legacy
 //  and poses as a stable interface now.
 static inline
@@ -147,13 +157,52 @@ s_zsys_vprintf_hint (int hint, const char *format, va_list argptr);
 //  Not threadsafe, so call only from main thread. Safe to call multiple
 //  times. Returns global CZMQ context.
 
+#if defined (__UNIX__)
+// mutex for pthread_once to run the init function only once in a process
+static pthread_once_t init_all_mutex_var = PTHREAD_ONCE_INIT;
+
+// handler to initialize mutexes one time in multi threaded env
+static void zsys_initialize_mutex() {
+    ZMUTEX_INIT (s_mutex);
+    ZMUTEX_INIT (s_init_mutex);
+}
+
+// handler to detect fork condition and cleanup the stale context inherited from parent process
+static void zsys_pthread_at_fork_handler(void) {
+    // re-initialize mutexes
+    ZMUTEX_INIT (s_init_mutex);
+    ZMUTEX_INIT (s_mutex);
+    // call cleanup
+    zsys_cleanup();
+}
+#endif
+
 void *
 zsys_init (void)
 {
+#if defined (__UNIX__)
+    //To avoid two inits at same time
+    pthread_once(&init_all_mutex_var, zsys_initialize_mutex);
+
     if (s_initialized) {
         assert (s_process_ctx);
         return s_process_ctx;
     }
+
+    ZMUTEX_LOCK (s_init_mutex);
+#endif
+
+    // Doing this again here... to ensure that after mutex wait if the thread 2 gets execution, it will
+    // will get the context right away
+    if (s_initialized) {
+        assert (s_process_ctx);
+#if defined (__UNIX__)
+        // unlock the mutex before returning the context
+        ZMUTEX_UNLOCK(s_init_mutex);
+#endif
+        return s_process_ctx;
+    }
+
     //  Pull process defaults from environment
     if (getenv ("ZSYS_IO_THREADS"))
         s_io_threads = atoi (getenv ("ZSYS_IO_THREADS"));
@@ -208,10 +257,15 @@ zsys_init (void)
 
     zsys_catch_interrupts ();
 
+#if defined (__WINDOWS__)
     ZMUTEX_INIT (s_mutex);
+#endif
     s_sockref_list = zlist_new ();
     if (!s_sockref_list) {
         zsys_shutdown ();
+#if defined (__UNIX__)
+        ZMUTEX_UNLOCK(s_init_mutex);
+#endif
         return NULL;
     }
     srandom ((unsigned) time (NULL));
@@ -224,7 +278,12 @@ zsys_init (void)
 #endif
     s_initialized = true;
 
+#if defined (__UNIX__)
     atexit (zsys_shutdown);
+    pthread_atfork(NULL, NULL, &zsys_pthread_at_fork_handler);
+    //don't hold the lock because some of the function will call zsys_init again
+    ZMUTEX_UNLOCK(s_init_mutex);
+#endif
 
     //  The following functions call zsys_init(), so they MUST be called after
     //  s_initialized is set in order to avoid an infinite recursion
@@ -239,6 +298,11 @@ zsys_init (void)
     else
         zsys_set_ipv6_mcast_address ("ff02:0:0:0:0:0:0:1");
 
+    if (getenv ("ZSYS_IPV4_MCAST_ADDRESS"))
+        zsys_set_ipv4_mcast_address (getenv ("ZSYS_IPV4_MCAST_ADDRESS"));
+    else
+        zsys_set_ipv4_mcast_address (NULL);
+
 
     if (getenv ("ZSYS_LOGIDENT"))
         zsys_set_logident (getenv ("ZSYS_LOGIDENT"));
@@ -269,6 +333,11 @@ zsys_init (void)
     else
         zsys_set_thread_name_prefix (s_thread_name_prefix);
 
+    if (getenv ("ZSYS_THREAD_NAME_PREFIX_STR"))
+        zsys_set_thread_name_prefix_str (getenv ("ZSYS_THREAD_NAME_PREFIX"));
+    else
+        zsys_set_thread_name_prefix_str (s_thread_name_prefix_str);
+
     return s_process_ctx;
 }
 
@@ -276,10 +345,10 @@ zsys_init (void)
 void
 zsys_shutdown (void)
 {
-    if (!s_initialized)
+    if (!s_initialized || s_shutting_down)
         return;
 
-    s_initialized = false;
+    s_shutting_down = true;
 
     //  The atexit handler is called when the main function exits;
     //  however we may have zactor threads shutting down and still
@@ -293,10 +362,6 @@ zsys_shutdown (void)
     if (busy)
         zclock_sleep (200);
 
-    //  Close logsender socket if opened (don't do this in critical section)
-    if (s_logsender)
-        zsock_destroy (&s_logsender);
-
     //  No matter, we are now going to shut down
     //  Print the source reference for any sockets the app did not
     //  destroy properly.
@@ -316,6 +381,10 @@ zsys_shutdown (void)
     zlist_destroy (&s_sockref_list);
     ZMUTEX_UNLOCK (s_mutex);
 
+    //  Close logsender socket if opened (don't do this in critical section)
+    if (s_logsender)
+        zsock_destroy (&s_logsender);
+
     if (s_open_sockets == 0)
     {
       zmq_term(s_process_ctx);
@@ -324,6 +393,7 @@ zsys_shutdown (void)
       s_thread_sched_policy = -1;
       s_thread_priority = -1;
       s_thread_name_prefix = -1;
+      strcpy (s_thread_name_prefix_str, "0");
       s_max_sockets = 1024;
       s_max_msgsz = INT_MAX;
       s_file_stable_age_msec = S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC;
@@ -355,8 +425,47 @@ zsys_shutdown (void)
 #if defined (__UNIX__)
     closelog ();                //  Just to be pedantic
 #endif
+
+    s_initialized = false;
+    s_shutting_down = false;
 }
 
+#if defined (__UNIX__)
+//  Restores all CZMQ global state to initial values
+static void
+zsys_cleanup (void)
+{
+    ZMUTEX_LOCK (s_init_mutex);
+    s_process_ctx = NULL;
+    zsys_interrupted = 0;
+    zctx_interrupted = 0;
+
+    s_first_time = true;
+    handle_signals = true;
+
+    s_initialized = false;
+
+    s_io_threads = 1;
+    s_max_sockets = 1024;
+    s_max_msgsz = INT_MAX;
+    s_linger = 0;
+    s_sndhwm = 1000;
+    s_rcvhwm = 1000;
+    s_pipehwm = 1000;
+    s_ipv6 = 0;
+    s_interface = NULL;
+    s_ipv6_address = NULL;
+    s_ipv6_mcast_address = NULL;
+    s_auto_use_fd = 0;
+    s_logident = NULL;
+    s_logstream = NULL;
+    s_logsystem = false;
+    s_logsender = NULL;
+
+    s_open_sockets = 0;
+    ZMUTEX_UNLOCK (s_init_mutex);
+}
+#endif
 
 //  --------------------------------------------------------------------------
 //  Get a new ZMQ socket, automagically creating a ZMQ context if this is
@@ -462,11 +571,13 @@ zsys_sockname (int socktype)
         "XPUB", "XSUB", "STREAM",
         "SERVER", "CLIENT",
         "RADIO", "DISH",
-        "SCATTER", "GATHER"
+        "SCATTER", "GATHER", "DGRAM"
     };
     //  This array matches ZMQ_XXX type definitions
     assert (ZMQ_PAIR == 0);
-#if defined (ZMQ_SCATTER)
+#if defined (ZMQ_DGRAM)
+    assert (socktype >= 0 && socktype <= ZMQ_DGRAM);
+#elif defined (ZMQ_SCATTER)
     assert (socktype >= 0 && socktype <= ZMQ_SCATTER);
 #elif defined (ZMQ_DISH)
     assert (socktype >= 0 && socktype <= ZMQ_DISH);
@@ -1029,8 +1140,8 @@ zsys_vprintf (const char *format, va_list argptr)
 SOCKET
 zsys_udp_new (bool routable)
 {
-    //  We haven't implemented multicast yet
-    assert (!routable);
+    //IPV6 Multicast not implemented yet so only allow routable if IPv4
+    assert ((routable && !zsys_ipv6 ()) || !routable);
     SOCKET udpsock;
     int type = SOCK_DGRAM;
 #ifdef CZMQ_HAVE_SOCK_CLOEXEC
@@ -1071,6 +1182,25 @@ zsys_udp_new (bool routable)
                     (char *) &on, sizeof (on)) == SOCKET_ERROR)
         zsys_socket_error ("setsockopt (SO_REUSEPORT)");
 #endif
+
+    //  Only set TLL for IPv4
+    if (routable && zsys_mcast_ttl () > 1) {
+        int ttl = zsys_mcast_ttl ();
+        if (setsockopt (udpsock, IPPROTO_IP, IP_MULTICAST_TTL, (const char *) &ttl,
+                        sizeof (ttl))
+            == SOCKET_ERROR)
+            zsys_socket_error ("setsockopt (IP_MULTICAST_TTL)");
+    }
+
+    //  TODO
+    //  Set TLL for IPv6
+    /*if (routable && zsys_ipv6 ()) {
+       int ttl = zsys_mcast_ttl ();
+        if (setsockopt (udpsock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+                (char *) &ttl, sizeof (ttl))
+            == SOCKET_ERROR)
+            zsys_socket_error ("setsockopt (IP_MULTICAST_TTL)");
+    }*/
     return udpsock;
 }
 
@@ -1510,6 +1640,63 @@ zsys_thread_name_prefix ()
 }
 
 
+//  --------------------------------------------------------------------------
+//  Configure the string prefix to each thread created for the internal
+//  context's thread pool. This option is only supported on Linux.
+//  If the environment variable ZSYS_THREAD_NAME_PREFIX_STR is defined, that
+//  provides the default.
+//  Note that this method is valid only before any socket is created.
+
+void
+zsys_set_thread_name_prefix_str (const char *prefix)
+{
+    size_t prefix_len = 0;
+
+    if (!prefix)
+        return;
+    prefix_len = strlen (prefix);
+    if (prefix_len == 0 || prefix_len > sizeof (s_thread_name_prefix_str) - 1)
+        return;
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    //  If the app is misusing this method, burn it with fire
+    if (s_open_sockets)
+        zsys_error ("zsys_set_thread_name_prefix() is not valid after"
+                " creating sockets");
+    assert (s_open_sockets == 0);
+    strcpy(s_thread_name_prefix_str, prefix);
+#if defined (ZMQ_THREAD_NAME_PREFIX) && defined (ZMQ_BUILD_DRAFT_API) && \
+    ((ZMQ_VERSION_MAJOR > 4) || \
+        ((ZMQ_VERSION_MAJOR >= 4) && ((ZMQ_VERSION_MINOR > 3) || \
+            ((ZMQ_VERSION_MINOR >= 3) && (ZMQ_VERSION_PATCH >= 3)))))
+    zmq_ctx_set_ext (s_process_ctx, ZMQ_THREAD_NAME_PREFIX, s_thread_name_prefix_str, sizeof (s_thread_name_prefix_str));
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+//  --------------------------------------------------------------------------
+//  Return ZMQ_THREAD_NAME_PREFIX_STR option.
+const char *
+zsys_thread_name_prefix_str ()
+{
+    size_t prefix_len = sizeof (s_thread_name_prefix_str);
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+#if defined (ZMQ_THREAD_NAME_PREFIX) && defined (ZMQ_BUILD_DRAFT_API) && \
+    ((ZMQ_VERSION_MAJOR > 4) || \
+        ((ZMQ_VERSION_MAJOR >= 4) && ((ZMQ_VERSION_MINOR > 3) || \
+            ((ZMQ_VERSION_MINOR >= 3) && (ZMQ_VERSION_PATCH >= 3)))))
+    zmq_ctx_get_ext (s_process_ctx, ZMQ_THREAD_NAME_PREFIX, s_thread_name_prefix_str, &prefix_len);
+#else
+    (void) prefix_len;
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+    return s_thread_name_prefix_str;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Adds a specific CPU to the affinity list of the ZMQ context thread pool.
 //  This option is only supported on Linux.
@@ -1873,6 +2060,58 @@ zsys_ipv6 (void)
     return s_ipv6;
 }
 
+//  Test if ipv6 is available on the system. The only way to reliably
+//  check is to actually open a socket and try to bind it. (ported from
+//  libzmq)
+
+bool
+zsys_ipv6_available (void)
+{
+#if defined(__WINDOWS__) && (_WIN32_WINNT < 0x0600)
+    return 0;
+#else
+    int rc, ipv6 = 1;
+    struct sockaddr_in6 test_addr;
+
+    memset (&test_addr, 0, sizeof (test_addr));
+    test_addr.sin6_family = AF_INET6;
+    inet_pton (AF_INET6, "::1", &(test_addr.sin6_addr));
+
+    SOCKET fd = socket (AF_INET6, SOCK_STREAM, IPPROTO_IP);
+    if (fd == INVALID_SOCKET)
+        ipv6 = 0;
+    else {
+#if defined(__WINDOWS__)
+        setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, (const char *) &ipv6,
+                    sizeof (int));
+        rc = setsockopt (fd, IPPROTO_IPV6, IPV6_V6ONLY, (const char *) &ipv6,
+                         sizeof (int));
+        if (rc == SOCKET_ERROR)
+            ipv6 = 0;
+        else {
+            rc = bind (fd, (struct sockaddr *) &test_addr, sizeof (test_addr));
+            if (rc == SOCKET_ERROR)
+                ipv6 = 0;
+        }
+        closesocket (fd);
+#else
+        setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, &ipv6, sizeof (int));
+        rc = setsockopt (fd, IPPROTO_IPV6, IPV6_V6ONLY, &ipv6, sizeof (int));
+        if (rc != 0)
+            ipv6 = 0;
+        else {
+            rc = bind (fd, (struct sockaddr *) (&test_addr),
+                       sizeof (test_addr));
+            if (rc != 0)
+                ipv6 = 0;
+        }
+        close (fd);
+#endif
+    }
+
+    return ipv6;
+#endif // _WIN32_WINNT < 0x0600
+}
 
 //  --------------------------------------------------------------------------
 //  Set network interface name to use for broadcasts, particularly zbeacon.
@@ -1943,6 +2182,31 @@ zsys_set_ipv6_mcast_address (const char *value)
     assert (s_ipv6_mcast_address);
 }
 
+//  --------------------------------------------------------------------------
+//  Set IPv4 multicast address to use for sending zbeacon messages. By default
+//  IPv4 multicast is NOT used. If the environment variable
+//  ZSYS_IPV4_MCAST_ADDRESS is set, use that as the default IPv4 multicast
+//  address. Calling this function or setting ZSYS_IPV4_MCAST_ADDRESS
+//  will enable IPv4 zbeacon messages.
+
+void zsys_set_ipv4_mcast_address (const char *value)
+{
+    zsys_init ();
+    freen (s_ipv4_mcast_address);
+    s_ipv4_mcast_address = value ? strdup (value) : NULL;
+    assert (!value || s_ipv4_mcast_address);
+}
+
+//  --------------------------------------------------------------------------
+//  Set multicast TTL
+
+void zsys_set_mcast_ttl (const unsigned char value)
+{
+    zsys_init ();
+    s_mcast_ttl = value;
+    assert (s_mcast_ttl);
+}
+
 
 //  --------------------------------------------------------------------------
 //  Return IPv6 multicast address to use for sending zbeacon, or
@@ -1954,6 +2218,23 @@ zsys_ipv6_mcast_address (void)
     return s_ipv6_mcast_address ? s_ipv6_mcast_address : "ff02:0:0:0:0:0:0:1";
 }
 
+//  --------------------------------------------------------------------------
+//  Return IPv4 multicast address to use for sending zbeacon, or
+//  NULL if none was set.
+
+const char *zsys_ipv4_mcast_address (void)
+{
+    return s_ipv4_mcast_address;
+}
+
+//  --------------------------------------------------------------------------
+//  Get multicast TTL
+
+unsigned char zsys_mcast_ttl ()
+{
+    return s_mcast_ttl;
+}
+
 
 //  --------------------------------------------------------------------------
 //  Configure the automatic use of pre-allocated FDs when creating new sockets.
@@ -2248,6 +2529,9 @@ zsys_set_logsystem (bool logsystem)
 static void
 s_log (char loglevel, char *string)
 {
+    if (!s_initialized)
+        zsys_init ();
+
 #if defined (__UNIX__)
 #   if defined (__UTYPE_ANDROID)
     int priority = ANDROID_LOG_INFO;
@@ -2400,7 +2684,10 @@ zsys_test (bool verbose)
     zsys_init();
     zsys_shutdown();
     zsys_init();
-
+#ifdef CZMQ_BUILD_DRAFT_API
+    // just check if we can check for ipv6
+    zsys_ipv6_available();
+#endif
 
     //  @selftest
     zsys_catch_interrupts ();
@@ -2432,6 +2719,7 @@ zsys_test (bool verbose)
     zsys_set_thread_sched_policy (-1);
     zsys_set_thread_name_prefix (0);
     assert (0 == zsys_thread_name_prefix());
+    assert (streq ("0", zsys_thread_name_prefix_str()));
     zsys_thread_affinity_cpu_add (0);
     zsys_thread_affinity_cpu_remove (0);
     zsys_set_zero_copy_recv(0);
@@ -2672,6 +2960,8 @@ zsys_test (bool verbose)
     zsys_set_max_msgsz (-1);
     assert (zsys_max_msgsz () == 2000);
 
+    // cleanup log_sender
+    zsys_set_logsender(NULL);
 
 #if defined (__WINDOWS__)
     zsys_shutdown();
