@@ -23,6 +23,22 @@
 
 #include "czmq_classes.h"
 
+#if defined (__WINDOWS__) && !defined (HAVE_UUID)
+#   define HAVE_UUID 1
+#endif
+#if defined (__UTYPE_OSX) && !defined (HAVE_UUID)
+#   define HAVE_UUID 1
+#endif
+#if defined (HAVE_UUID) || defined (__UTYPE_FREEBSD)
+#   if defined (__UTYPE_FREEBSD) || defined (__UTYPE_NETBSD) || defined(__UTYPE_OPENBSD)
+#       include <uuid.h>
+#   elif defined __UTYPE_HPUX
+#       include <dce/uuid.h>
+#   elif defined (__UNIX__)
+#       include <uuid/uuid.h>
+#   endif
+#endif
+
 //  Structure of our class
 
 struct _zuuid_t {
@@ -47,7 +63,7 @@ zuuid_new (void)
     assert (sizeof (uuid) == ZUUID_LEN);
     UuidCreate (&uuid);
     zuuid_set (self, (byte *) &uuid);
-#elif defined (HAVE_UUID) && !defined (__UTYPE_FREEBSD)
+#elif defined (HAVE_UUID)
     uuid_t uuid;
     assert (sizeof (uuid) == ZUUID_LEN);
     uuid_generate (uuid);
